;last update: 2014-10-17 by Andrea La Camera
;+
;At this point, the overlap value depends on three different "sources":
; (a) the one coming from the Enclosed Energy of the PSF
; (b) the one coming from the difference (np - n) (i.e. size of the psf - size of the domain)
; (c) the user-defined value
;When called, the routine computes (a) and (b), checks (c) and
;then verifies which value is the largest one.
;Finally the routine prints warning messages if requested by the
;keyword ask_confirm=1
;-

pro patch_check_overlap, par, parent_ID, auto_overlap, check_overlap, VERBOSE=verbose

if KEYWORD_SET(verbose) then verb=1 else verb=0
check_overlap = 0
;(a) compute EE, find max(EE(psf_k)) 
;
;(b) find np and n
  delta = 0
  max_np = 0
  if par.psf_type EQ 0 then psf=readfits((*par.psf_filename)[0],/SILENT)
  for k=0,par.cols*par.rows-1 do begin
     if par.psf_type EQ 1 then begin
        psf_curr=readfits((*par.psf_filename)[k],/SILENT)
     endif else psf_curr=psf[*,*,k]
     NP=(size(psf_curr))[1]
     MP=(size(psf_curr))[2]
     delta_k=patch_psf_ee(psf_curr, par.threshold)
     if (delta_k GT delta) then delta = delta_k
     if (max([NP,MP]) GT max_np) then max_np=max([NP,MP])
  endfor
                                ;max size of the domains
  max_n = max([par.domx,par.domy])
  
;(c) check par.overlap (the user-defined value)
;    and find the largest value
  auto_overlap = max([delta, ROUND(((max_np-max_n)>0)/2)])
  if (par.overlap LT auto_overlap) then check_overlap = 1
  
;print warnings if necessary....
  if (verb AND check_overlap) then begin
     a=dialog_message(["THE OVERLAP IS TOO SMALL!", $
                       "The user-defined overlap must be greater than",$
                       "the auto value ("+strtrim(auto_overlap,1)+")",$
                       " ",$
                       "Select OK to continue and accept this number",$
                       "or Cancel to return to the GUI."],$
                      dialog_parent=parent_ID, /CANCEL, /INFO)
     if (a EQ 'Cancel') then check_overlap=0 
  endif
end
