;PATCH_GUI.PRO
;This routine manages the Graphical User Interface (GUI)
;and its events for the software PATCH 
;Andrea La Camera (DIBRIS, Univ. of Genova), November 2011
;last modified: 2014-10-17
;---------
;


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
pro patch_gui_event, event
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  
  COMMON image, img, header, N, M, a_dim, Nori, Mori
  COMMON background, bg
  COMMON working_dir, wd
  COMMON output, out, res_array, residual ; reconstructed object with original dimension Nori x Mori
  
  
  WIDGET_CONTROL, event.ID, GET_UVALUE=choice
  WIDGET_CONTROL, event.TOP, GET_UVALUE=state
; handle a kill request (considered as a cancel event).
  if tag_names(event, /STRUCTURE_NAME) eq 'WIDGET_KILL_REQUEST' then begin
     widget_control, event.top, /DESTROY
  endif
  
  case choice of
     'img_load' : begin
        state.par.img_filename=dialog_pickfile(dialog_parent=event.top, $
                                               FILTER=['*.fits', '*.fit', '*.fts', '*.*'], $
                                               PATH=wd, GET_PATH=new_wd, $
                                               /MUST_EXIST, title="Load a FITS file (must be a 2d array)")
        if state.par.img_filename EQ '' then return
        wd=new_wd                       
        a=patch_load_image(state, event.top, /RESET, /VERBOSE)
        if a then patch_show_image, state, /redraw else return
    end
     'img_define' : begin
        if state.par.img_filename EQ '' then return
        result=patch_img_define("SUB-DOMAINS DEFINITION", state, group_leader=event.top)
        state.par=result
        if (N GT (size(img))[1]) OR (M GT (size(img))[2]) then begin
           img = zero_padding(img, N, M)
        endif
        info=["filename: "+state.par.img_filename, $
              "size: "+strtrim(N,1)+' x '+strtrim(M,1), $
              "domains ("+strtrim(state.par.rows*state.par.cols,1)+"): size "+$
              strtrim(state.par.domx,1)+' x '+strtrim(state.par.domy,1), $
              "current domain: none"]
                                ;show info
        WIDGET_CONTROL, state.id.img_info, SET_VALUE=info
        patch_show_image, state, /REDRAW
     end
     'iarea' : begin
        if state.par.img_filename EQ '' then return
                                ;print, event.x, event.y, event.clicks, event.press, event.modifiers 
        if (event.press EQ 1) AND (event.modifiers EQ 0) then begin
                                ;select a sub-domain
           if state.par.rows*state.par.cols GT 1 then begin
              patch_show_image, state ;Clean area
              x0=(FIX(event.x/state.par.img_zoom-state.par.overlap)>0<(N-1)) / state.par.domx
              y0=(FIX(event.y/state.par.img_zoom-state.par.overlap)>0<(M-1)) / state.par.domy
                                ;draw the box around selected sub-region
              plots, (indgen(state.par.domx+2*state.par.overlap)+x0*state.par.domx)*state.par.img_zoom, $
                     (y0*state.par.domy)*state.par.img_zoom, /DEV, $
                     color='FFFFFF'x, LINESTYLE=0, thick=2
              plots, (indgen(state.par.domx+2*state.par.overlap)+x0*state.par.domx)*state.par.img_zoom, $
                     ((y0+1)*state.par.domy+2*state.par.overlap-2)*state.par.img_zoom, /DEV, $
                     color='FFFFFF'x, LINESTYLE=0, thick=2                  
              plots, (x0*state.par.domx)*state.par.img_zoom, $
                     (indgen(state.par.domy+2*state.par.overlap)+y0*state.par.domy)*state.par.img_zoom, $
                     /DEV, color='FFFFFF'x, LINESTYLE=0, thick=2       
              plots, ((x0+1)*state.par.domx+2*state.par.overlap-2)*state.par.img_zoom, $
                     (indgen(state.par.domy+2*state.par.overlap)+y0*state.par.domy)*state.par.img_zoom, $
                     /DEV, color='FFFFFF'x, LINESTYLE=0, thick=2
              if ((state.par.domx+state.par.domy)*state.par.img_zoom) LT 51 then begin
                 xyouts, ((x0+0.5)*state.par.domx+state.par.overlap)*state.par.img_zoom, $
                         ((y0+0.47)*state.par.domy+state.par.overlap)*state.par.img_zoom, $
                         strtrim(y0*state.par.cols+x0,1), $
                         /DEVICE, charsize=1.2, ALIGNMENT=0.5, charthick=2.
              endif
                                ;show info 
              info=["filename: "+state.par.img_filename, $
                    "size: "+strtrim(N,1)+' x '+strtrim(M,1), $
                    "domains ("+strtrim(state.par.rows*state.par.cols,1)+"): size "+$
                    strtrim(state.par.domx,1)+' x '+strtrim(state.par.domy,1), $
                   "current domain: "+strtrim(y0*state.par.cols+x0,1)]
              WIDGET_CONTROL, state.id.img_info, SET_VALUE=info

           endif
        endif
        if (event.press EQ 4) then begin
                                ;select options (zoom, scale, color, etc)
           result=patch_img_options("OPTIONS", state, group_leader=event.top)
           if result.img_zoom EQ state.par.img_zoom then begin
              state.par.img_scale = result.img_scale
              state.par.img_color = result.img_color
              patch_show_image, state ; REDRAW is not necessary
           endif else begin
              state.par.img_zoom  = result.img_zoom
              state.par.img_scale = result.img_scale
              state.par.img_color = result.img_color
              patch_show_image, state, /REDRAW ; REDRAW is necessary
           endelse
        endif
     end
     'psf_type': begin
        state.par.psf_type=event.value
                                ;reset info of psf
        widget_control, state.id.psf_info, set_value=strarr(4)
        patch_set_gui, state
     end   
     'psf_load' : begin
        nb_of_psfs = state.par.cols*state.par.rows
        if state.par.psf_type EQ 0 then begin
           res=dialog_pickfile(FILTER=['*.fits', '*.fit', '*.fts', '*.*'], /MUST_EXIST, $
                               path=wd, get_path=new_wd, $
                               title="Load the FITS file containing the "+strtrim(nb_of_psfs,1)+" PSFs.", $
                               dialog_parent=event.top)
           if res EQ '' then return
           wd=new_wd
           (*state.par.psf_filename)[0]=res
           a=patch_load_psfs(state, event.top)                     
        endif
        if state.par.psf_type EQ 1 then begin
           res=dialog_pickfile(FILTER=['*.fits', '*.fit', '*.fts', '*.*'], $
                               path=wd, get_path=new_wd, $
                               /MUST_EXIST, /MULTIPLE_FILES, $
                               title="Load "+strtrim(nb_of_psfs,1)+$
                               " FITS files (must be 2d arrays)", dialog_parent=event.top)
           if (n_elements(res) EQ 0) OR (res[0] EQ '') then return
           if n_elements(res) NE nb_of_psfs then begin
              a=dialog_message("Wrong number of PSFs: "+strtrim(nb_of_psfs,1)+$
                               " files expected.", dialog_parent=event.top, /ERROR)
              return
           endif
           wd=new_wd
                                ;load psfs
           (*state.par.psf_filename)=res
           a=patch_load_psfs(state, event.top) 
        endif 
     end
     'threshold': state.par.threshold = float(event.value)/100.
     'auto_overlap': begin
        if state.par.img_filename EQ '' then return
        print, "AUTO OVERLAP calculating ..."
        patch_check_overlap, state.par, event.top, auto_overlap, check_overlap, VERBOSE=0
        state.par.overlap=auto_overlap
        WIDGET_CONTROL, state.id.overlap, SET_VALUE=state.par.overlap
        print, "...done!"
     end
     'overlap' : begin
        state.par.overlap=event.value 
        print, state.par.overlap
     end
     'check_overlap' : begin
        if state.par.img_filename EQ '' then return
        print, "CHECK OVERLAP calculating ... "
        patch_check_overlap, state.par, event.top, auto_overlap, check_overlap, /VERBOSE
        if check_overlap then begin
           state.par.overlap=auto_overlap
           WIDGET_CONTROL, state.id.overlap, SET_VALUE=state.par.overlap
        endif
        print, "...done!"
     end
     'redraw' : patch_show_image, state, /REDRAW
     'bg_choice': begin
        state.par.bg_choice = event.value
        patch_set_gui, state
     end
     'bg' : begin
        state.par.bg=event.value 
        bg = make_array(N,M,value=state.par.bg)
     end
     'bg_file' : begin
        state.par.bg_file=dialog_pickfile(dialog_parent=event.top, $
                                          FILTER=['*.fits', '*.fit', '*.fts', '*.*'], $
                                          PATH=wd, GET_PATH=new_wd, $
                                          /MUST_EXIST, title="Load a FITS file (must be a 2d array)")
        if state.par.bg_file EQ '' then return
        a=patch_load_background(state, event.top, /VERB)
        wd=new_wd  
     end 
     'sigma_ron': state.par.sigma_ron = event.value 
     'gain' : state.par.gain = event.value 
     'method' : begin
        state.par.method=event.index
        patch_set_gui, state
     end
     'stop_crit' : begin
        state.par.stop_crit = event.index
        patch_set_gui, state
     end
     'tol'  : state.par.tol = event.value
     'mask_choice': begin
        state.par.mask_choice = event.select
        patch_set_gui, state
     end
     'mask_filename': begin
        state.par.mask_filename=dialog_pickfile(dialog_parent=event.top, $
                                                FILTER=['*.fits', '*.fit', '*.fts', '*.*'], $
                                                PATH=wd, GET_PATH=new_wd, $
                                                /MUST_EXIST, title="Load a FITS file (must be a 2d array)")
        if state.par.mask_filename EQ '' then return
        wd=new_wd
        mask=readfits(state.par.mask_filename, /SILENT)
        dim_mask_1=(size(mask))[1]
        info=["filename: "+state.par.mask_filename, $
              "size: "+strtrim((size(mask))[1],1)+' x '+strtrim((size(mask))[2],1)]
                                ;show info
        WIDGET_CONTROL, state.id.mask_info, SET_VALUE=info
     end
     'it_choice': begin
        state.par.it_choice=event.value
        patch_set_gui, state
                                ;compute the NoI table here (1)
        patch_noi_table_compute, event.value, state.par
     end        
     'tot_iter': begin
        state.par.tot_iter=event.value
                                ;compute the NoI table also here (2)
        patch_noi_table_compute, 0, state.par
     end
     'it4': begin
        widget_control, state.id.it4, get_value=dummy
        if dummy[event.x,event.y] LE 0 then begin
           a=dialog_message("Insert positive (>0) values!", /ERROR)
        endif
        dummy=float(rotate(dummy,7))
        state.par.it4=dummy
                                ;compute the NoI table also here (3)
        patch_noi_table_compute, 1, state.par
     end
     'itedit':begin
        if n_elements(*state.par.tab_iter) NE state.par.cols*state.par.rows then begin
                                ;compute the NoI table also here (4)
           patch_noi_table_compute, state.par.it_choice, state.par
        endif
        res=patch_noi_table(state.par,GROUP_LEADER=event.top)
        (*state.par.tab_iter)=(*res.tab_iter)
        ;print, '--> EXIT : ', *state.par.tab_iter
     end
     'go' : begin
        if n_elements(img) EQ 0 then begin
           a=dialog_message("Input image undefined!", /ERROR)
           return
        endif
        if n_elements(bg) EQ 0 then begin
           a=dialog_message("Background undefined! Insert positive (>0) values", /ERROR)           
           return
        endif
        if (*state.par.psf_filename)[0] EQ '' then return
         ;;;;; MODIFICA PER RMSE in ogni sotto-dominio
                                ;a= dialog_pickfile(dialog_parent=event.top, FILTER=['*.fits'], $
                                ;                  path=wd, get_path=new_wd, $
                                ;                  /MUST_EXIST, title="CARICA L'OGGETTO NORMALIZZATO")
                                ;print, a
                                ;if a NE '' then begin
                                ;   fits_read, a, obj
                                ;   N_tmp=(size(obj))[1]
                                ;   M_tmp=(size(obj))[2]
                                ;                       ;check if zero_padding is neccessary...
                                ;   if (state.par.cols*state.par.domx GT N_tmp) OR $
                                ;      (state.par.rows*state.par.domy GT M_tmp) then begin
                                ;      obj = zero_padding(obj, N, M)
                                ;   endif
                                ;endif else obj=make_array(N,M)
                                ;help, obj
         ;;;;;;;fine modifica
        t0=systime(1)
        print, "Please, wait..."
        WIDGET_CONTROL, state.id.run_deconv, SET_VALUE="   .... wait ....  "
        WIDGET_CONTROL, state.id.run_deconv, SENSITIVE=0
        patch_deconv, state.par ;, obj ;;;; aggiunto obj !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
        WIDGET_CONTROL, state.id.run_deconv, SET_VALUE=" RUN DECONVOLUTION "
        WIDGET_CONTROL, state.id.run_deconv, /SENSITIVE
        print, "...done!"
        print, "Total deconvolution time (s) = ", systime(1)-t0
        print, "Average time for each sub-domain = ", $
               (systime(1)-t0)/(state.par.rows*state.par.cols) 
        patch_show_out, state, /REDRAW ; REDRAW is necessary
     end
     'save_out' : begin
        res=dialog_pickfile(dialog_parent=event.top, $
                            FILTER=['*.fits', '*.fit', '*.fts'], $
                            path=wd, get_path=new_wd, $
                            /OVERWRITE_PROMPT, /WRITE, $
                            title="Save as a FITS file (must be a 2d array)")
        if res EQ '' then return else state.par.out_filename=res
        wd=new_wd
        if n_elements(Nori) EQ 0 then begin
           info=["filename: "+state.par.out_filename, $
                 "size:     not yet defined "]
        endif else begin
           info=["filename: "+state.par.out_filename, $
                 "    size: "+strtrim(Nori,1)+' x '+strtrim(Mori,1)]
        endelse      
                                ;show info (OK or ERROR)
        WIDGET_CONTROL, state.id.out_info, SET_VALUE=info
        if n_elements(out) GT 0 then begin
                                ;save output in FITS file 
           writefits, state.par.out_filename, out, header
        endif
     end
     'show_out': begin
        if n_elements(out) GT 0 then patch_show_out, state, /REDRAW ; REDRAW is necessary
     end
     'header': begin
        if n_elements(header) GT 0 then xdispstr, header, TITLE="FITS HEADER"
     end
     'save_res' : begin
        res=dialog_pickfile(dialog_parent=event.top, $
                            FILTER=['*.fits', '*.fit', '*.fts'], $
                            path=wd, get_path=new_wd, $
                            /OVERWRITE_PROMPT, /WRITE, $
                            title="Save as a FITS file (must be a 2d array)")
        if res EQ '' then return else state.par.res_filename=res
        wd=new_wd
        if n_elements(Nori) EQ 0 then begin
           info=["filename: "+state.par.res_filename, $
                 "size:     not yet defined "]
        endif else begin
           info=["filename: "+state.par.res_filename, $
                 "    size: "+strtrim(Nori,1)+' x '+strtrim(Mori,1)]
        endelse      
                                ;show info (OK or ERROR)
        WIDGET_CONTROL, state.id.res_info, SET_VALUE=info
        if n_elements(residual) GT 0 then begin
                                ;save output in FITS file 
           writefits, state.par.res_filename, residual, header
        endif
     end
     'show_res': begin
        if n_elements(residual) GT 0 then patch_show_res, state, /REDRAW ; REDRAW is necessary
     end
     'stat_res': begin  
        if n_elements(residual) GT 0 then begin
            window,/free,title="Residual Statistics - HISTOGAUSS RESULT"
            histogauss, residual, stat, charsize=1.8
        endif
     end
     'oarea': begin
        if state.par.img_filename EQ '' then return
        if (event.press EQ 4) then begin
                                ;select options (zoom, scale, color, etc)
           result=patch_img_options("OPTIONS", state, group_leader=event.top)
           if result.img_zoom EQ state.par.img_zoom then begin
              state.par.img_scale = result.img_scale
              state.par.img_color = result.img_color
              patch_show_out, state ; REDRAW is not necessary
           endif else begin
              state.par.img_zoom  = result.img_zoom
              state.par.img_scale = result.img_scale
              state.par.img_color = result.img_color
              patch_show_out, state, /REDRAW ; REDRAW is necessary
           endelse
        endif
     end
     'load_par': begin
        a=dialog_pickfile(dialog_parent=event.top, FILTER=['*.sav'], $
                          path=wd, get_path=new_wd, $
                          /MUST_EXIST, /READ)
        if a EQ '' then return
        wd=new_wd
        restore, a, /VERBOSE
                                ;restore all parameters and set the gui::: Read structure is "PARAM"
        state.par=param
                                ; load image and define img, N, M
        a=patch_load_image(state, event.top, /VERBOSE)
        if a EQ 0 then return
                                ;check if zero_padding is neccessary...
        if (state.par.cols*state.par.domx GT N) OR $
           (state.par.rows*state.par.domy GT M) then begin
           N=state.par.cols*state.par.domx
           M=state.par.rows*state.par.domy
           img = zero_padding(img, N, M)
        endif
                                ;show image
        patch_show_image, state, /REDRAW
                                ;info about image
        info=["filename: "+state.par.img_filename, $
              "size: "+strtrim(N,1)+' x '+strtrim(M,1), $
              "domains ("+strtrim(state.par.rows*state.par.cols,1)+"): size "+$
              strtrim(state.par.domx,1)+' x '+strtrim(state.par.domy,1), $
              "current domain: none"]
                                ;show info
        WIDGET_CONTROL, state.id.img_info, SET_VALUE=info
                                ; load PSF
        WIDGET_CONTROL, state.id.psf_type, SET_VALUE=state.par.psf_type
        a=patch_load_psfs(state, event.top)
        WIDGET_CONTROL, state.id.threshold, SET_VALUE=fix(state.par.threshold*100.)
        WIDGET_CONTROL, state.id.overlap, SET_VALUE=state.par.overlap
                                ;set "DECONVOLUTION" tab
        if state.par.bg_choice EQ 0 then begin
           WIDGET_CONTROL, state.id.bg, SET_VALUE=state.par.bg
           bg = make_array(N,M,value=state.par.bg)
        endif else begin
           a=patch_load_background(state, event.top, /VERBOSE)
        endelse
        WIDGET_CONTROL, state.id.bg_choice, SET_VALUE=state.par.bg_choice
        WIDGET_CONTROL, state.id.sigma_ron, SET_VALUE=state.par.sigma_ron
        WIDGET_CONTROL, state.id.gain, SET_VALUE=state.par.gain
        WIDGET_CONTROL, state.id.method, SET_COMBOBOX_SELECT=state.par.method
        WIDGET_CONTROL, state.id.stop_crit, SET_COMBOBOX_SELECT=state.par.stop_crit
        WIDGET_CONTROL, state.id.tol, SET_VALUE=state.par.tol
        
        
        
        WIDGET_CONTROL, state.id.tot_iter, SET_VALUE=state.par.tot_iter
        info=["filename: "+state.par.out_filename, $
              "    size: "+strtrim(Nori,1)+' x '+strtrim(Mori,1)]      
        WIDGET_CONTROL, state.id.out_info, SET_VALUE=info 
        info=["filename: "+state.par.res_filename, $
              "    size: "+strtrim(Nori,1)+' x '+strtrim(Mori,1)]      
        WIDGET_CONTROL, state.id.res_info, SET_VALUE=info 
        patch_set_gui, state
     end
;     'load_rec': begin
;        print, event.value
;     end
     'save_par': begin
        a=dialog_pickfile(dialog_parent=event.top, FILTER=['*.sav'], $
                          path=wd, get_path=new_wd, $
                          /WRITE)
        if a EQ '' then return
        wd=new_wd
        param=state.par
        save, param, FILENAME=a, /VERBOSE
     end   
;   'help' : begin
;         a=dialog_message("NOT YET IMPLEMENTED", /INFO, dialog_parent=event.top)
                                ;XDISPLAYFILE, nomefile, DONE_BUTTON="Done", Title="PATCH Help"
;      end
     'web'  : begin
        online_help, book="website.html"
     end
     'about' : a=dialog_message(["                 PATCH                  ", $
                                 "                 version 0.7                 ", $
                                 "           written by Andrea La Camera       ", $
                                 "         (DIBRIS, University of Genova)      ", $
                                 "                  February 2014              "], $
                                /INFO, title="About this software", $
                                dialog_parent=event.top)
     'exit' : begin
        widget_control, event.top, /DESTROY
        return
     end
     
     else: return
  endcase                       ;end of main CASE 
  
; write the GUI state structure
  WIDGET_CONTROL, event.top, SET_UVALUE=state
end 


;;;;;;;;;;;;;;;;;;
pro patch_gui
;;;;;;;;;;;;;;;;;;
  
  COMMON image, img, header, N, M, a_dim
  COMMON working_dir, wd
  COMMON output, out, res_array, residual            ; reconstructed object with original dimension Nori x Mori
  
  cd, current=wd
  device, decompose=0
  
;Definition of the structure "STATE", made of two sub-structures: "ID" and "PAR"
  
  id = { img_define:0L, img_info:0L, $
         ibase:0L, ikill:0L, iarea:0L, $
         psf_type : 0L, psf_info:0L, text:0L, threshold:0L, $
         overlap:0L, $
         method:0L, stop_crit:0L, tol:0L, tot_iter:0L, $
         mask_choice:0L, mask_filename:0L, mask_info:0L, $
         bg:0L, bg_choice:0L, bg_file:0L, bg_info:0L, $
         it_choice:0L, it4:0L, itedit:0L, $
         sigma_ron:0L, gain:0L, run_deconv:0L, $
         obase:0L, okill:0L, oarea:0L, $
         out_info:0L, res_info:0L  }
  
  par = { img_filename:"", img_scale:0, img_zoom:1., $
          img_color:0, psf_type:0, psf_filename:PTR_NEW(strarr(4)),  $
          threshold:0.95, $
          domx:-1, domy:-1, rows:1, cols:1, overlap:0, $
          method:0, stop_crit:0, tol:1e-4, tot_iter:1L, out_filename:"", $
          mask_choice:0, mask_filename:"", $
          bg:0., bg_file:'', bg_choice: 0B, $
          it_choice:0, it4:[[1L,1L],[1L,1L]], tab_iter:PTR_NEW(1), $
          sigma_ron:10., gain:1., res_filename:"" }
  
  state={id:id, par:par}
  
;Graphical User Interface starts here 
  base_id = widget_base(TITLE="PATCH - Graphical User Interface", MODAL=0, /COL, $ 
                        GROUP_LEADER=group, MBAR=bar)
  menu1=WIDGET_BUTTON(bar, value="File", /MENU)
  menu1b10=WIDGET_BUTTON(menu1, value="Load parameters...", uvalue="load_par")
;OPENR, lun, file_which('_filelist.txt'), /GET_LUN
;load_rec_desc = '1\Load recent '
;line = ''
;WHILE NOT EOF(lun) DO BEGIN 
;  READF, lun, line 
;  load_rec_desc = [load_rec_desc, '0\'+line] 
;ENDWHILE
;close,lun
;FREE_LUN, lun
;load_rec_desc=[load_rec_desc,'2\Clear menu']
;menu1b11=CW_PDMENU(menu1, load_rec_desc, /RETURN_NAME, uvalue="load_rec", /MBAR)
  menu1b20=WIDGET_BUTTON(menu1, value="Save parameters...", uvalue="save_par")
  menu1b99=WIDGET_BUTTON(menu1, value="Exit", uvalue="exit")
;menu2=WIDGET_BUTTON(bar, value="Image", /MENU)
;menu2b1=WIDGET_BUTTON(menu2, value="Options", /MENU)
;menu2b1a=WIDGET_BUTTON(menu2b1, value="Zoom", uvalue
  
  menu0=WIDGET_BUTTON(bar, value="Help", /MENU)
;menu0b1=WIDGET_BUTTON(menu0, value="Help", uvalue="help")
  menu0b2=WIDGET_BUTTON(menu0, value="Online tutorial", uvalue="web")
  menu0b3=WIDGET_BUTTON(menu0, value="About", uvalue="about")
  
  
  basetab=WIDGET_TAB(base_id, location=0, uvalue='')
;--------------------------------------------------
  baseA=WIDGET_BASE(basetab, /COL, TITLE=' INPUT ', /ALIGN_LEFT)
;--------------------------------------------------
  baseAA=WIDGET_BASE(baseA, /ROW, /ALIGN_LEFT)
  base_left=WIDGET_BASE(baseAA, /COL, /ALIGN_LEFT)
  base_img=WIDGET_BASE(base_left, /COL, /ALIGN_LEFT, /FRAME, Title="Blurred image")
  
  base_img1=WIDGET_BASE(base_img, /ROW, /ALIGN_LEFT)
  label = WIDGET_LABEL(base_img1, value="      Blurred image    --> ")
  button = WIDGET_BUTTON(base_img1,value="Load...", UVALUE="img_load", xsize=60, ysize=30)
  state.id.img_info=CW_FIELD(base_img, TITLE='Information: ', $
                             VALUE=' ', /NOEDIT, /COL, YSIZE=4, xsize=50)
  state.id.img_define = WIDGET_BUTTON(base_img, value="Define sub-domains", uvalue="img_define")
  label= WIDGET_LABEL(base_left, value="  ")
  
  base_psf=WIDGET_BASE(base_left, /COL, /ALIGN_LEFT, /FRAME, Title="Point Spread Functions")
  base_psf1=WIDGET_BASE(base_psf, /ROW, /ALIGN_LEFT)
  label = WIDGET_LABEL(base_psf1, value="Point Spread Functions --> ", /ALIGN_LEFT)
  button = WIDGET_BUTTON(base_psf1,value="Load...", UVALUE="psf_load", xsize=60, ysize=30)
  state.id.psf_type=CW_BGROUP(base_psf,["PSFs are stored in a single FITS file.",$
                                        "PSFs are stored in separate FITS files. "],    $
                              ROW=2, SET_VALUE=state.par.psf_type, UVALUE='psf_type', /EXCLUSIVE) 
  state.id.text = WIDGET_TEXT(base_psf, value=[" ", " ", " ", " "], YSIZE=4, FRAME=0)             
  state.id.psf_info=CW_FIELD(base_psf, TITLE='Information: ', $
                             VALUE=' ', /NOEDIT, /COL, YSIZE=4, xsize=50)
  state.id.threshold = WIDGET_SLIDER(base_psf, value=fix(state.par.threshold*100), $
                                     xsize=100, uvalue='threshold', $
                                     title="EE Threshold (percentage)")
  
  
  base_right=WIDGET_BASE(baseAA, /COL, /ALIGN_TOP, /FRAME)
  state.id.ibase=WIDGET_BASE(base_right, /ALIGN_TOP)
  a_dim=512
  state.id.ikill=WIDGET_DRAW(state.id.ibase, XSIZE=a_dim, YSIZE=a_dim, /scroll, $
                             X_SCROLL_SIZE=a_dim, Y_SCROLL_SIZE=a_dim, KEYBOARD_EVENTS=2, $
                             /button_events, uvalue='iarea', /ALIGN_CENTER, $
                             TOOLTIP='Load an image...', RETAIN=1)

  baseAB=WIDGET_BASE(baseA, /ROW, /ALIGN_LEFT, /FRAME, Title="Overlap definition")
  base_overlap1 = WIDGET_BASE(baseAB, /COL, /ALIGN_LEFT)
  label= WIDGET_LABEL(base_overlap1, value="Overlapping region: ")
  label= WIDGET_LABEL(base_overlap1, value=" *) Automatically computed")
  button = WIDGET_BUTTON(base_overlap1,value="AUTO", UVALUE='auto_overlap', xsize=70, ysize=30, /ALIGN_CENTER)

  base_overlap2 = WIDGET_BASE(baseAB, /COL, /ALIGN_LEFT)
  state.id.overlap = CW_FIELD(base_overlap2, title="*) User-defined value (pixels):", $
                              /INTEGER, uvalue='overlap', /RETURN_EVENTS, $
                              value=state.par.overlap,xsize=9,/ROW)     
  button = WIDGET_BUTTON(base_overlap2,value="CHECK", UVALUE='check_overlap', xsize=70, ysize=30, /ALIGN_CENTER)
    
  base_overlap3 = WIDGET_BASE(baseAB, /COL, /ALIGN_LEFT)
   label= WIDGET_LABEL(base_overlap3, value="Press REDRAW or click on the")
  label= WIDGET_LABEL(base_overlap3, value="image to see the result.")
  button = WIDGET_BUTTON(base_overlap3,value="REDRAW", UVALUE='redraw', xsize=70, ysize=30, /ALIGN_CENTER)


  
;--------------------------------------------------
  baseB=WIDGET_BASE(basetab, /COL, TITLE=' DECONVOLUTION ', /ALIGN_LEFT)
;--------------------------------------------------
  base_dec=WIDGET_BASE(baseB, /ROW, /ALIGN_TOP, /FRAME)
  base_dec_left = WIDGET_BASE(base_dec, /COL, /ALIGN_LEFT, /FRAME)
;;;;
  base_bg = WIDGET_BASE(base_dec_left, /ROW)
  state.id.bg_choice = CW_BGROUP(base_bg,["Costant value [ADU/pixel]", $
                                          "from FITS file           "], $
                                 /COL, LABEL_TOP="Sky Background: ", $
                                 SET_VALUE=state.par.bg_choice, UVALUE='bg_choice', /EXCLUSIVE) 
  base_bg1 = WIDGET_BASE(base_bg, /COL, /ALIGN_CENTER, /BASE_ALIGN_TOP)
  state.id.bg=CW_FIELD(base_bg1, title="", $
                       VALUE=state.par.bg, xsize=10, /ROW, /FLOAT, /ALL_EVENTS, $
                       uvalue="bg")
  state.id.bg_file = WIDGET_BUTTON(base_bg1,value="Load...", UVALUE="bg_file", $
                                   xsize=60, ysize=30)
  state.id.bg_info=CW_FIELD(base_bg, TITLE='Information: ', $
                            VALUE=' ', /NOEDIT, /COL, YSIZE=4, xsize=35)
;;;;                                 
  state.id.sigma_ron=CW_FIELD(base_dec_left, title="Read-Out Noise value [e-/pixel]:", $
                              VALUE=state.par.sigma_ron, xsize=10, /ROW, /FLOAT, /ALL_EVENTS, $
                              uvalue="sigma_ron")             
  
  state.id.gain=CW_FIELD(base_dec_left,      title="GAIN value [e-/ADU]:            ", $
                         VALUE=state.par.gain, xsize=10, /ROW, /FLOAT, /ALL_EVENTS, $
                         uvalue="gain")             
;;;
  base_dec_right = WIDGET_BASE(base_dec, /COL, /ALIGN_LEFT, /FRAME)
  base_dec_right1 = WIDGET_BASE(base_dec_right, /ROW, /ALIGN_LEFT)
  label=WIDGET_LABEL(base_dec_right1, value="Restoration method:")         
  method=["Richardson-Lucy (RL)",  "Scaled Gradient Projection (SGP) " ] 
  state.id.method = WIDGET_COMBOBOX(base_dec_right1, value=method, uvalue='method')
  base_dec_right2 = WIDGET_BASE(base_dec_right, /ROW, /ALIGN_LEFT)
  label=WIDGET_LABEL(base_dec_right2, value="Stopping rule:     ")         
  state.id.stop_crit = WIDGET_COMBOBOX(base_dec_right2, $
                                       value=["                                 "], $
                                       uvalue='stop_crit')
  base_dec_right3 = WIDGET_BASE(base_dec_right, /ROW, /ALIGN_LEFT)
  state.id.tol = CW_FIELD(base_dec_right3, title = "Tolerance: ", uvalue='tol', $
                          value=state.par.tol, xsize=15, /ROW, /FLOAT, /ALL_EVENTS)
  base_dec_right4 = WIDGET_BASE(base_dec_right, /ROW, /ALIGN_LEFT)
  state.id.mask_choice = CW_BGROUP(base_dec_right4, ["Apply a mask to the result of each iteration"], $
                                   uvalue='mask_choice', set_value=state.par.mask_choice, $
                                   /NONEXCLUSIVE)
  state.id.mask_filename = WIDGET_BUTTON(base_dec_right4,value="Load...", UVALUE="mask_filename", $
                                         xsize=60, ysize=30)
  state.id.mask_info = CW_FIELD(base_dec_right, TITLE='Information: ', $
                                VALUE=' ', /NOEDIT, /COL, YSIZE=2, xsize=35)
;;;
  base_dec2=WIDGET_BASE(baseB, /COL, /ALIGN_TOP, /FRAME)
  state.id.it_choice = CW_BGROUP(base_dec2,["Fixed value for each sub-dom.         ", $
                                            "Bilinear interpolation                ", $ 
                                            "User-defined [manually edit the table]"], $
                                 /ROW, LABEL_TOP="Number of iterations (NoI) for each sub-domain: ", $
                                 SET_VALUE=state.par.it_choice, UVALUE='it_choice', $
                                 /EXCLUSIVE, /NO_RELEASE) 
  base_iter = WIDGET_BASE(base_dec2, /ROW)
  ;base_iter1 = WIDGET_BASE(base_iter, /COL, /ALIGN_CENTER, /BASE_ALIGN_TOP)
  dummy = widget_label(base_iter, value="     ")
  state.id.tot_iter=CW_FIELD(base_iter, title="", $
                             VALUE=state.par.tot_iter, xsize=8, /COL, /LONG, /ALL_EVENTS, $
                             uvalue="tot_iter")
  dummy = widget_label(base_iter, value="             Corner values: ")
  state.id.it4=WIDGET_TABLE(base_iter, value=rotate(state.par.it4,7),COLUMN_LABELS=["Left","Right"], $
                            ROW_LABELS=["Top","Bottom"], $
                            /EDITABLE, uvalue="it4",COLUMN_WIDTH=40, ROW_HEIGHTS=27, $
                           scr_xsize=155, scr_ysize=90)
  base_iter2 = WIDGET_BASE(base_dec2, /ROW)
  dummy = widget_label(base_iter2, value="                                                ")
  state.id.itedit=WIDGET_BUTTON(base_iter2, value="Show/Edit the NoI table", $
                                uvalue="itedit", xsize=180, ysize=30)

;;;
;base_out=WIDGET_BASE(baseB, /ROW, /ALIGN_LEFT, /FRAME)
  base_run=WIDGET_BASE(baseB, /COL, /ALIGN_CENTER, /FRAME)
  dummy=WIDGET_LABEL(base_run, value= "   ")
  state.id.run_deconv=WIDGET_BUTTON(base_run, value=" RUN DECONVOLUTION ", uvalue='go', $
                                    xsize=250, ysize=50)
  dummy=WIDGET_LABEL(base_run, value= "   ")
  
;--------------------------------------------------
  baseC=WIDGET_BASE(basetab, /ROW, TITLE=' OUTPUT ', /ALIGN_LEFT)
;--------------------------------------------------
  base_left=WIDGET_BASE(baseC, /COL, /ALIGN_LEFT)
  base_out=WIDGET_BASE(base_left, /COL, /ALIGN_LEFT, /FRAME)
  label=WIDGET_LABEL(base_out, value="Reconstructed object:")
  base_out2 = WIDGET_BASE(base_out, /ROW, /ALIGN_LEFT)
  button = WIDGET_BUTTON(base_out2, value="Save as...", uvalue="save_out")
  button = WIDGET_BUTTON(base_out2, value="Show", uvalue="show_out")
  button = WIDGET_BUTTON(base_out2, value="Display FITS header", uvalue="header")
  state.id.out_info=CW_FIELD(base_out, TITLE='Information: ', $
                             VALUE=' ', /NOEDIT, /COL, YSIZE=2, xsize=40)
  
  base_res=WIDGET_BASE(base_left, /COL, /ALIGN_LEFT, /FRAME)
  label=WIDGET_LABEL(base_res, value="Residual:")
  base_res2 = WIDGET_BASE(base_res, /ROW, /ALIGN_LEFT)
  button = WIDGET_BUTTON(base_res2, value="Save as...", uvalue="save_res")
  button = WIDGET_BUTTON(base_res2, value="Show", uvalue="show_res")
  button = WIDGET_BUTTON(base_res2, value="Statistics", uvalue="stat_res")
  state.id.res_info=CW_FIELD(base_res, TITLE='Information: ', $
                             VALUE=' ', /NOEDIT, /COL, YSIZE=2, xsize=40)

  base_right=WIDGET_BASE(baseC, /COL, /ALIGN_TOP, /FRAME)
  state.id.obase=WIDGET_BASE(base_right, /ALIGN_TOP)
  a_dim=512
  state.id.okill=WIDGET_DRAW(state.id.obase, XSIZE=a_dim, YSIZE=a_dim, /scroll, $
                             X_SCROLL_SIZE=a_dim, Y_SCROLL_SIZE=a_dim, KEYBOARD_EVENTS=2, $
                             /button_events, uvalue='oarea', /ALIGN_CENTER, $
                             TOOLTIP='', RETAIN=1)
  
; -----------------------------------------------------------------------
;Standard buttons
; -----------------------------------------------------------------------
  base2  = WIDGET_BASE(base_id, FRAME=1, /ROW, /ALIGN_LEFT)
  help_id = WIDGET_BUTTON(base2, VALUE='HELP',UVALUE='web')
  cancel_id  = WIDGET_BUTTON(base2, VALUE='EXIT',UVALUE='exit')
;restore_id = WIDGET_BUTTON(base2, VALUE='RESTORE PARAMETERS',UVALUE='restore')
;save_id    = WIDGET_BUTTON(base2, VALUE='SAVE PARAMETERS',UVALUE='save')
  
  WIDGET_CONTROL, cancel_id, /CANCEL_BUTTON
;WIDGET_CONTROL, save_id, /DEFAULT_BUTTON
  
  WIDGET_CONTROL, base_id, /REALIZE
  WIDGET_CONTROL, state.id.ikill, GET_VALUE=dID
  state.id.iarea=dID
  WIDGET_CONTROL, base_id, SET_UVALUE=state
  patch_set_gui, state
  XMANAGER, 'patch_gui', base_id, GROUP_LEADER=group
  
end
